@@ -7,13 +7,13 @@
 #ifndef _LIBIME_LIBIME_CORE_UTILS_P_H_
 #define _LIBIME_LIBIME_CORE_UTILS_P_H_
 
-#include "endian_p.h"
 #include <chrono>
 #include <cstdint>
 #include <iostream>
 #include <string>
 #include <string_view>
 #include <vector>
+#include "endian_p.h"
 
 namespace libime {
 
@@ -194,6 +194,19 @@ inline int millisecondsTill(T t0) {
         .count();
 }
 
+struct StringHash {
+    using hash_type = std::hash<std::string_view>;
+    using is_transparent = void;
+
+    std::size_t operator()(const char *str) const { return hash_type{}(str); }
+    std::size_t operator()(std::string_view str) const {
+        return hash_type{}(str);
+    }
+    std::size_t operator()(const std::string &str) const {
+        return hash_type{}(str);
+    }
+};
+
 } // namespace libime
 
 #endif // _LIBIME_LIBIME_CORE_UTILS_P_H_
