@@ -10,7 +10,9 @@
 #include <functional>
 #include <iterator>
 #include <limits>
+#include <list>
 #include <memory>
+#include <span>
 #include <stdexcept>
 #include <string>
 #include <string_view>
@@ -80,6 +82,7 @@ public:
     mutable std::vector<SentenceResult> candidatesToCursor_;
     mutable std::unordered_set<std::string> candidatesToCursorSet_;
     std::vector<fcitx::ScopedConnection> conn_;
+    std::list<WordNode> contextWords_;
 
     size_t alignCursorToNextSegment() const {
         FCITX_Q();
@@ -248,9 +251,17 @@ public:
                 totalPinyinLength += item.encodedPinyin_.size() / 2;
             }
         }
-        if (!isAllSingleWord && !hasCustom && totalPinyinLength > 4) {
-            return LearnWordResult::Ignored;
+
+        FCITX_Q();
+        if (!hasCustom) {
+            if ((!isAllSingleWord && totalPinyinLength > 4)) {
+                return LearnWordResult::Ignored;
+            }
+            if (ime_->model()->containsNonUnigram(q->selectedWords())) {
+                return LearnWordResult::Ignored;
+            }
         }
+
         for (auto &s : selected_) {
             for (auto &item : s) {
                 if (item.type_ == SelectedPinyinType::Separator) {
@@ -526,16 +537,19 @@ State PinyinContext::state() const {
     FCITX_D();
     auto *model = d->ime_->model();
     State state = model->nullState();
-    if (!d->selected_.empty()) {
-        for (const auto &s : d->selected_) {
-            for (const auto &item : s) {
-                if (item.word_.word().empty()) {
-                    continue;
-                }
-                State temp;
-                model->score(state, item.word_, temp);
-                state = std::move(temp);
+    for (const auto &word : d->contextWords_) {
+        State temp;
+        model->score(state, word, temp);
+        state = std::move(temp);
+    }
+    for (const auto &s : d->selected_) {
+        for (const auto &item : s) {
+            if (item.word_.word().empty()) {
+                continue;
             }
+            State temp;
+            model->score(state, item.word_, temp);
+            state = std::move(temp);
         }
     }
     return state;
@@ -552,21 +566,9 @@ void PinyinContext::update() {
         d->clearCandidates();
     } else {
         size_t start = 0;
-        auto *model = d->ime_->model();
-        State state = model->nullState();
+        State state = this->state();
         if (!d->selected_.empty()) {
             start = d->selected_.back().back().offset_;
-
-            for (auto &s : d->selected_) {
-                for (auto &item : s) {
-                    if (item.word_.word().empty()) {
-                        continue;
-                    }
-                    State temp;
-                    model->score(state, item.word_, temp);
-                    state = std::move(temp);
-                }
-            }
         }
         SegmentGraph newGraph;
         if (auto spProfile = d->matchState_.shuangpinProfile()) {
@@ -991,6 +993,44 @@ void PinyinContext::learn() {
     }
 }
 
+void PinyinContext::setContextWords(
+    const std::vector<std::string> &contextWords) {
+    FCITX_D();
+    d->contextWords_.clear();
+    appendContextWords(contextWords);
+}
+
+void PinyinContext::clearContextWords() {
+    FCITX_D();
+    d->contextWords_.clear();
+}
+
+void PinyinContext::appendContextWords(
+    const std::vector<std::string> &contextWords) {
+    FCITX_D();
+
+    size_t needed = LanguageModel::maxOrder() - 1;
+
+    for (const auto &word :
+         std::span{contextWords}.last(std::min(contextWords.size(), needed))) {
+        d->contextWords_.push_back(
+            WordNode(word, d->ime_->model()->index(word)));
+    }
+    while (d->contextWords_.size() > needed) {
+        d->contextWords_.pop_front();
+    }
+}
+
+std::vector<std::string> PinyinContext::contextWords() const {
+    FCITX_D();
+    std::vector<std::string> words;
+    words.reserve(d->contextWords_.size());
+    for (const auto &word : d->contextWords_) {
+        words.push_back(word.word());
+    }
+    return words;
+}
+
 bool PinyinContext::learnWord() { return false; }
 
 PinyinIME *PinyinContext::ime() const {
