@@ -5,11 +5,8 @@
  */
 
 #include "pinyindata.h"
-#include "pinyinencoder.h"
 #include <algorithm>
 #include <cstdint>
-#include <fcitx-utils/log.h>
-#include <fcitx-utils/stringutils.h>
 #include <iterator>
 #include <optional>
 #include <string>
@@ -17,6 +14,10 @@
 #include <unordered_set>
 #include <utility>
 #include <vector>
+#include <fcitx-utils/log.h>
+#include <fcitx-utils/stringutils.h>
+#include "pinyindata_p.h"
+#include "pinyinencoder.h"
 
 namespace libime {
 
@@ -137,11 +138,54 @@ getInnerSegment() {
             {"dian", {"di", "an"}},     {"cuan", {"cu", "an"}},
             {"chuang", {"chu", "ang"}}, {"chuan", {"chu", "an"}},
             {"chuai", {"chu", "ai"}},   {"biang", {"bi", "ang"}},
+
+            {"qiao", {"qia", "o"}},     {"niao", {"nia", "o"}},
+            {"liao", {"lia", "o"}},     {"zhuo", {"zhu", "o"}},
+            {"diao", {"dia", "o"}},     {"shao", {"sha", "o"}},
+            {"xiao", {"xia", "o"}},     {"zhua", {"zhu", "a"}},
+            {"shuo", {"shu", "o"}},     {"shua", {"shu", "a"}},
+            {"zhao", {"zha", "o"}},     {"jiao", {"jia", "o"}},
+            {"chuo", {"chu", "o"}},     {"chua", {"chu", "a"}},
+            {"chao", {"cha", "o"}},
+
         };
 
     return innerSegment;
 }
 
+const InnerSegmentMap &getInnerSegmentV2() {
+    static const InnerSegmentMap innerSegment = []() {
+        InnerSegmentMap innerSegmentV2;
+        for (const auto &[key, value] : getInnerSegment()) {
+            innerSegmentV2[key].push_back(value);
+        }
+
+        for (const auto &newItem : std::vector<
+                 std::pair<std::string, std::pair<std::string, std::string>>>{
+                 {"qiao", {"qia", "o"}},
+                 {"niao", {"nia", "o"}},
+                 {"liao", {"lia", "o"}},
+                 {"zhuo", {"zhu", "o"}},
+                 {"diao", {"dia", "o"}},
+                 {"shao", {"sha", "o"}},
+                 {"xiao", {"xia", "o"}},
+                 {"zhua", {"zhu", "a"}},
+                 {"shuo", {"shu", "o"}},
+                 {"shua", {"shu", "a"}},
+                 {"zhao", {"zha", "o"}},
+                 {"jiao", {"jia", "o"}},
+                 {"chuo", {"chu", "o"}},
+                 {"chua", {"chu", "a"}},
+                 {"chao", {"cha", "o"}},
+             }) {
+            innerSegmentV2[newItem.first].push_back(newItem.second);
+        }
+        return innerSegmentV2;
+    }();
+
+    return innerSegment;
+}
+
 inline bool operator==(const PinyinEntry &a, const PinyinEntry &b) {
     return a.pinyin() == b.pinyin() && a.initial() == b.initial() &&
            a.final() == b.final() && a.flags() == b.flags();
