@@ -4,6 +4,16 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
+#include <array>
+#include <cstddef>
+#include <iostream>
+#include <memory>
+#include <ostream>
+#include <sstream>
+#include <string>
+#include <string_view>
+#include <vector>
+#include <fcitx-utils/log.h>
 #include "libime/core/historybigram.h"
 #include "libime/core/lattice.h"
 #include "libime/core/userlanguagemodel.h"
@@ -13,14 +23,6 @@
 #include "libime/pinyin/pinyinencoder.h"
 #include "libime/pinyin/pinyinime.h"
 #include "testdir.h"
-#include <array>
-#include <cstddef>
-#include <fcitx-utils/log.h>
-#include <iostream>
-#include <memory>
-#include <ostream>
-#include <sstream>
-#include <string_view>
 
 using namespace libime;
 
@@ -131,8 +133,7 @@ int main() {
     c.type("n");
     for (const auto &candidate : c.candidates()) {
         for (const auto *node : candidate.sentence()) {
-            const auto &pinyin =
-                static_cast<const PinyinLatticeNode *>(node)->encodedPinyin();
+            const auto &pinyin = node->as<PinyinLatticeNode>().encodedPinyin();
             std::cout << node->word();
             if (!pinyin.empty()) {
                 std::cout << " " << PinyinEncoder::decodeFullPinyin(pinyin);
@@ -236,6 +237,75 @@ int main() {
         c.type("shounihao");
         FCITX_ASSERT(c.candidatesToCursorSet().count("✋你好") > 0);
     }
+    {
+        c.clear();
+        c.type("er");
+        c.selectCustom(2, "");
+        FCITX_ASSERT(c.selected());
+        FCITX_ASSERT(c.selectedSentence() == "");
+        FCITX_ASSERT(c.selectedWords().size() == 1);
+        FCITX_ASSERT(c.selectedWords().front().empty());
+        FCITX_ASSERT(c.selectedWordsWithPinyin().size() == 1);
+    }
+
+    // Check that context words can change prediction.
+    {
+        {
+            c.clear();
+            c.type("ta");
+            size_t i = 0;
+            for (const auto &candidate : c.candidatesToCursor()) {
+                if (candidate.toString() == "她") {
+                    break;
+                }
+                i++;
+            }
+            FCITX_ASSERT(i > 0) << i;
+        }
+        {
+            c.clear();
+            c.setContextWords({"我", "不", "知道"});
+            FCITX_ASSERT(c.contextWords() ==
+                         std::vector<std::string>{"不", "知道"});
+            c.setContextWords({"谁", "他"});
+            FCITX_ASSERT(c.contextWords() ==
+                         std::vector<std::string>{"谁", "他"});
+            c.appendContextWords({"爱"});
+            FCITX_ASSERT(c.contextWords() ==
+                         std::vector<std::string>{"他", "爱"});
+            c.type("ta");
+            size_t i = 0;
+            for (const auto &candidate : c.candidatesToCursor()) {
+                if (candidate.toString() == "她") {
+                    break;
+                }
+                i++;
+            }
+            FCITX_ASSERT(i == 0) << i;
+        }
+    }
+
+    {
+        c.clear();
+        c.clearContextWords();
+        FCITX_ASSERT(!ime.model()->history().containsBigram("他", "爱"));
+        c.type("taai");
+        size_t i = 0;
+        for (const auto &candidate : c.candidatesToCursor()) {
+            if (candidate.toString() == "他爱") {
+                break;
+            }
+            i++;
+        }
+        FCITX_ASSERT(i < c.candidatesToCursor().size());
+        c.selectCandidatesToCursor(i);
+
+        FCITX_ASSERT(c.selected());
+        FCITX_ASSERT(c.selectedSentence() == "他爱");
+        c.learn();
+        c.clear();
+        FCITX_ASSERT(ime.model()->history().containsBigram("他", "爱"));
+    }
 
     return 0;
 }
