@@ -4,12 +4,15 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
+#include <cstddef>
+#include <iostream>
+#include <ostream>
+#include <sstream>
+#include <string_view>
+#include <fcitx-utils/log.h>
 #include "libime/pinyin/pinyindictionary.h"
 #include "libime/pinyin/pinyinencoder.h"
 #include "testdir.h"
-#include "testutils.h"
-#include <fcitx-utils/log.h>
-#include <sstream>
 
 using namespace libime;
 
